from django.conf import settings
from django.db import models
from django.utils.encoding import force_text
from . import forms
from .slugify import slugify, django_slugify
from .validators import ValidateSlug


class SlugPreviewField(models.SlugField):
    """
    Slug field that also displays a preview.
    """
    def __init__(self, *args, **kwargs):
        self.populate_from = kwargs.pop('populate_from', None)  # nice idea from django-autoslug, but used differently.
        self.always_update = kwargs.pop('always_update', False)
        self.slugify = kwargs.pop('slugify', slugify)
        self.url_format = kwargs.pop('url_format', None)

        if self.slugify != django_slugify:
            # This replaces the 'default_validators' setting at object level.
            self.default_validators = [ValidateSlug(self.slugify)]

        super(SlugPreviewField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        # Pass our custom settings to the form field
        defaults = {
            'form_class': forms.SlugPreviewField,
            'populate_from': self.populate_from,
            'always_update': self.always_update,
            'slugify': self.slugify,
            'url_format': self.url_format,
        }
        defaults.update(kwargs)
        return super(SlugPreviewField, self).formfield(**defaults)

    def pre_save(self, instance, add):
        """
        Auto-generate the slug if needed.
        """
        # get currently entered slug
        value = self.value_from_object(instance)
        slug = None

        # auto populate (if the form didn't do that already).
        # If you want unique_with logic, use django-autoslug instead.
        # This model field only allows parameters which can be passed to the form widget too.
        if self.populate_from and (self.always_update or not value):
            value = getattr(instance, self.populate_from)

        # Make sure the slugify logic is applied,
        # even on manually entered input.
        if value:
            value = force_text(value)
            slug = self.slugify(value)
            if self.max_length < len(slug):
                slug = slug[:self.max_length]

        # make the updated slug available as instance attribute
        setattr(instance, self.name, slug)
        return slug

    def south_field_triple(self):
        from south.modelsinspector import introspector
        path = "{0}.{1}".format(self.__class__.__module__, self.__class__.__name__)
        args, kwargs = introspector(self)
        return (path, args, kwargs)


# Avoid using AdminTextInputWidget
if 'django.contrib.admin' in settings.INSTALLED_APPS:
    from django.contrib.admin import options
    options.FORMFIELD_FOR_DBFIELD_DEFAULTS[SlugPreviewField] = {}



from __future__ import absolute_import
from django.utils.text import slugify as django_slugify

try:
    # awesome-slugify, unicode-slugify
    from slugify import slugify
except ImportError:
    # Fallback to Django ASCII-only support
    slugify = django_slugify



from __future__ import unicode_literals
import re

from django.conf import settings
from django.forms import widgets
from django.utils.html import format_html


class SlugPreviewWidget(widgets.TextInput):
    """
    Slug preview
    """
    class Media:
        css={
            'all': (
                'slug_preview/css/slug_preview.css',
            )
        }
        js=(
            'admin/js/vendor/jquery/jquery{}.js'.format('' if settings.DEBUG else '.min'),
            'admin/js/jquery.init.js',
            'admin/js/urlify.js',
            'admin/js/prepopulate{}.js'.format('' if settings.DEBUG else '.min'),
            'slug_preview/js/slug_preview.js',
        )

    def __init__(self, attrs=None, url_format=None):
        super(SlugPreviewWidget, self).__init__(attrs=attrs)
        self.url_format = url_format
        self.instance = None   # assigned via BoundSlugField

    def render(self, name, value, attrs=None, **kwargs):
        if attrs is None:
            attrs = {}
        input_tag = super(SlugPreviewWidget, self).render(name, value, attrs=attrs, **kwargs)
        return format_html('<kbd class="slugpreview">{0}</kbd>', self.render_preview(input_tag))

    def render_preview(self, input_tag):
        url_format = self.url_format
        if not url_format:
            # Late evaluation, to allow access to self.instance
            if self.instance is not None and hasattr(self.instance, 'get_absolute_url_format'):
                url_format = self.instance.get_absolute_url_format()
            else:
                url_format = u'/{slug}/'

        before, after = url_format.split('{slug}', 2)
        return format_html(
            '<span class="url-prefix">{before}</span>'     # noqa indent
            '<span class="url-slug">{input_tag}</span>'
            '<span class="url-suffix">{after}</span>',
            before=before,
            input_tag=input_tag,
            after=after,
        )



from django import forms
from django.forms import BoundField
from . import widgets
from .slugify import slugify, django_slugify
from .validators import ValidateSlug


class SlugPreviewFormMixin(object):
    """
    Form Mixin to add necessary integration for the SlugPreviewField.
    This extends the default ``form[field]`` interface that produces the BoundField for HTML templates.
    """
    def __getitem__(self, item):
        boundfield = super(SlugPreviewFormMixin, self).__getitem__(item)
        if isinstance(boundfield.field, SlugPreviewField):
            boundfield.__class__ = _upgrade_boundfield_class(boundfield.__class__)
        return boundfield


class BoundSlugField(BoundField):
    """
    An exta integration to pass information of the form to the widget.
    This is loaded via :class:`SlugPreviewFormMixin`
    """
    def as_widget(self, widget=None, attrs=None, only_initial=False):
        if not widget:
            widget = self.field.widget

        widget.instance = self.form.instance  # Widget needs ability to fill in the blanks.
        return super(BoundSlugField, self).as_widget(widget=widget, attrs=attrs, only_initial=only_initial)


UPGRADED_CLASSES = {}
def _upgrade_boundfield_class(cls):
    if cls is BoundField:
        return BoundSlugField
    elif issubclass(cls, BoundSlugField):
        return cls

    # When some other package also performs this same trick,
    # combine both classes on the fly. Avoid having to do that each time.
    # This is needed for django-parler
    try:
        return UPGRADED_CLASSES[cls]
    except KeyError:
        # Create once
        new_cls = type('BoundSlugField_{0}'.format(cls.__name__), (cls, BoundSlugField), {})
        UPGRADED_CLASSES[cls] = new_cls
        return new_cls


class SlugPreviewModelForm(SlugPreviewFormMixin, forms.ModelForm):
    """
    A default model form, configured with the :class:`SlugPreviewFormMixin`.
    """


class SlugPreviewField(forms.SlugField):
    """
    A Form field that displays the slug and preview.
    It requires the :class:`SlugPreviewFormMixin` to be present in the form.
    """
    widget = widgets.SlugPreviewWidget

    def __init__(self, *args, **kwargs):
        self.populate_from = kwargs.pop('populate_from', None)
        self.always_update = kwargs.pop('always_update', False)
        self.url_format = kwargs.pop('url_format', None)
        self.slugify = kwargs.pop('slugify', slugify)

        if self.slugify != django_slugify:
            # This replaces the 'default_validators' setting at object level.
            self.default_validators = [ValidateSlug(self.slugify)]

        super(SlugPreviewField, self).__init__(*args, **kwargs)
        self.widget.url_format = self.url_format

    def widget_attrs(self, widget):
        # Expose the form field settings to HTML
        attrs = super(SlugPreviewField, self).widget_attrs(widget)
        attrs.update({
            'data-populate-from': self.populate_from,
            'data-url-format': self.url_format,
            'data-always-update': self.always_update,
        })
        return attrs



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from .slugify import slugify


class ValidateSlug(object):
    """
    Test whether the given value is an accepted slug.

    The class can test against custom slug functions (e.g. awesome-slugify).
    It should be used instead of the standard slug validators,
    because these only accept what the standard Django slugify() can process.
    """
    slugify = slugify
    message = _('Enter a valid slug.')
    code = 'invalid'

    def __init__(self, slugify=None, message=None, code=None):
        if slugify is not None:
            self.slugify = slugify
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

    def __call__(self, value):
        if self.slugify(value) != value:
            raise ValidationError(self.message, code=self.code)
